--- app_dial.c.orig	2006-07-06 15:18:45.000000000 +0100
+++ app_dial.c	2006-07-07 13:04:54.000000000 +0100
@@ -162,6 +162,9 @@
 "           family/key is not specified.\n"
 "    r    - Indicate ringing to the calling party. Pass no audio to the calling\n"
 "           party until the called channel has answered.\n"
+"    s(n[:s])   - Spoof CLID on dialling channel to 'n' with an optional CLID\n"
+"           string of 's'. To clear CLID string but set CLID use s(1234:).\n"
+"           Spoof overrides all other CLID options.\n"
 "    S(x) - Hang up the call after 'x' seconds *after* the called party has\n"
 "           answered the call.\n"  	
 "    t    - Allow the called party to transfer the calling party by sending the\n"
@@ -214,6 +217,7 @@
 	OPT_CALLEE_MONITOR = (1 << 21),
 	OPT_CALLER_MONITOR = (1 << 22),
 	OPT_GOTO = (1 << 23),
+	OPT_CLID_SPOOF = (1 << 24),
 } dial_exec_option_flags;
 
 #define DIAL_STILLGOING			(1 << 30)
@@ -228,6 +232,7 @@
 	OPT_ARG_CALLEE_MACRO,
 	OPT_ARG_PRIVACY,
 	OPT_ARG_DURATION_STOP,
+	OPT_ARG_CLID_SPOOF,
 	/* note: this entry _MUST_ be the last one in the enum */
 	OPT_ARG_ARRAY_SIZE,
 } dial_exec_option_args;
@@ -252,6 +257,7 @@
 	AST_APP_OPTION('p', OPT_SCREENING),
 	AST_APP_OPTION_ARG('P', OPT_PRIVACY, OPT_ARG_PRIVACY),
 	AST_APP_OPTION('r', OPT_RINGBACK),
+	AST_APP_OPTION_ARG('s', OPT_CLID_SPOOF, OPT_ARG_CLID_SPOOF),
 	AST_APP_OPTION_ARG('S', OPT_DURATION_STOP, OPT_ARG_DURATION_STOP),
 	AST_APP_OPTION('t', OPT_CALLEE_TRANSFER),
 	AST_APP_OPTION('T', OPT_CALLER_TRANSFER),
@@ -766,6 +772,7 @@
 	int digit = 0, result = 0;
 	time_t start_time, answer_time, end_time;
 	struct ast_app *app = NULL;
+	char *clidspoof=NULL, *clidnamespoof=NULL;
 
 	char *parse;
 	AST_DECLARE_APP_ARGS(args,
@@ -817,6 +824,12 @@
 		dtmfcalling = parse;
 	}
 
+	if (ast_test_flag(&opts, OPT_CLID_SPOOF) && !ast_strlen_zero(opt_args[OPT_ARG_CLID_SPOOF])) {
+		parse = opt_args[OPT_ARG_CLID_SPOOF];
+		clidspoof = strsep(&parse, ":");
+		clidnamespoof = parse;
+	}
+
 	if (ast_test_flag(&opts, OPT_DURATION_LIMIT) && !ast_strlen_zero(opt_args[OPT_ARG_DURATION_LIMIT])) {
 		char *limit_str, *warning_str, *warnfreq_str;
 
@@ -992,7 +1005,7 @@
 	/* If a channel group has been specified, get it for use when we create peer channels */
 	outbound_group = pbx_builtin_getvar_helper(chan, "OUTBOUND_GROUP");
 
-	ast_copy_flags(peerflags, &opts, OPT_DTMF_EXIT | OPT_GO_ON | OPT_ORIGINAL_CLID | OPT_CALLER_HANGUP);
+	ast_copy_flags(peerflags, &opts, OPT_DTMF_EXIT | OPT_GO_ON | OPT_ORIGINAL_CLID | OPT_CALLER_HANGUP | OPT_CLID_SPOOF);
 	cur = args.peers;
 	do {
 		/* Remember where to start next time */
@@ -1021,7 +1034,7 @@
 				       OPT_CALLEE_TRANSFER | OPT_CALLER_TRANSFER |
 				       OPT_CALLEE_HANGUP | OPT_CALLER_HANGUP |
 				       OPT_CALLEE_MONITOR | OPT_CALLER_MONITOR |
-				       OPT_RINGBACK | OPT_MUSICBACK | OPT_FORCECLID);
+				       OPT_RINGBACK | OPT_MUSICBACK | OPT_FORCECLID | OPT_CLID_SPOOF);
 			ast_set2_flag(tmp, args.url, DIAL_NOFORWARDHTML);	
 		}
 		ast_copy_string(numsubst, number, sizeof(numsubst));
@@ -1100,12 +1113,19 @@
 			free(tmp->chan->cid.cid_ani);
 		tmp->chan->cid.cid_ani = NULL;
 
-		if (chan->cid.cid_num) 
-			tmp->chan->cid.cid_num = strdup(chan->cid.cid_num);
-		if (chan->cid.cid_name) 
-			tmp->chan->cid.cid_name = strdup(chan->cid.cid_name);
-		if (chan->cid.cid_ani) 
+		if (chan->cid.cid_ani)
 			tmp->chan->cid.cid_ani = strdup(chan->cid.cid_ani);
+
+		if(ast_test_flag(tmp,OPT_CLID_SPOOF))
+		{
+			tmp->chan->cid.cid_num = strdup(clidspoof ? clidspoof : "");
+			tmp->chan->cid.cid_name = strdup(clidnamespoof ? clidnamespoof : "");
+		} else {
+			if (chan->cid.cid_num) 
+				tmp->chan->cid.cid_num = strdup(chan->cid.cid_num);
+			if (chan->cid.cid_name) 
+				tmp->chan->cid.cid_name = strdup(chan->cid.cid_name);
+		}
 		
 		/* Copy language from incoming to outgoing */
 		ast_copy_string(tmp->chan->language, chan->language, sizeof(tmp->chan->language));
